using System.Collections;
using Firebase;
using Firebase.Auth;
using TMPro;
using UnityEngine;

namespace Assets
{
    public class AuthManager : MonoBehaviour
    {
        //Firebase variables
        [Header("Firebase")]
        public DependencyStatus dependencyStatus;
        public FirebaseAuth auth;    
        public FirebaseUser User;

        //Login variables
        [Header("Login")]
        public TMP_InputField emailLoginField;
        public TMP_InputField passwordLoginField;
        public TMP_Text warningLoginText;
        public TMP_Text confirmLoginText;

        //Register variables
        [Header("Register")]
        public TMP_InputField usernameRegisterField;
        public TMP_InputField emailRegisterField;
        public TMP_InputField passwordRegisterField;
        public TMP_InputField passwordRegisterVerifyField;
        public TMP_Text warningRegisterText;

        void Awake()
        {
            FirebaseApp.CheckAndFixDependenciesAsync().ContinueWith(task =>
            {
                dependencyStatus = task.Result;

                if (dependencyStatus == DependencyStatus.Available)
                {
                    InitFirebase();
                }
                else
                {
                    print("Firebase dependency error");
                }
            });
        }

        void InitFirebase()
        {
            auth = FirebaseAuth.DefaultInstance;
            print("Ready to " + auth);
        }

        public void LogIn()
        {
            StartCoroutine(DoLogIn(emailLoginField.text, passwordLoginField.text));
        }
        
        IEnumerator DoLogIn(string email, string password)
        {
            var logInTask = auth.SignInWithEmailAndPasswordAsync(email, password);

            yield return new WaitUntil(() => logInTask.IsCompleted);


            if (logInTask.Exception != null)
            {
                if (logInTask.Exception.GetBaseException() is not FirebaseException firebaseException) yield break;
                var errorCode = (AuthError)firebaseException.ErrorCode;

                var message = errorCode switch
                {
                    AuthError.MissingEmail => "Missing email",
                    AuthError.MissingPassword => "Missing password",
                    AuthError.WrongPassword => "Wrong password",
                    AuthError.InvalidEmail => "Invalid email",
                    AuthError.UserNotFound => "User not found",
                    _ => "Log in failed"
                };

                warningLoginText.text = message;
            }
            else
            {
                User = logInTask.Result;
                warningLoginText.text = "";
                confirmLoginText.text = "Log in";
            }
        }
        
        public void Create()
        {
            StartCoroutine(DoCreate(emailRegisterField.text, passwordRegisterField.text, usernameRegisterField.text));
        }

        IEnumerator DoCreate(string email, string password, string userName)
        {
            if (userName == "")
            {
                warningRegisterText.text = "Missing user name";
            }
            else if (password != passwordRegisterVerifyField.text)
            {
                warningRegisterText.text = "Not match";
            }
            else
            {
                var createTask = auth.CreateUserWithEmailAndPasswordAsync(email, password);
                yield return new WaitUntil(() => createTask.IsCompleted);

                if (createTask.Exception != null)
                {
                    if (createTask.Exception.GetBaseException() is not FirebaseException firebaseException) yield break;
                    var errorCode = (AuthError)firebaseException.ErrorCode;

                    var message = errorCode switch
                    {
                        AuthError.MissingEmail => "Missing email",
                        AuthError.MissingPassword => "Missing password",
                        AuthError.WeakPassword => "Wreak password",
                        AuthError.EmailAlreadyInUse => "Email used",
                        _ => "Create failed"
                    };

                    warningRegisterText.text = message;
                }
                else
                {
                    User = createTask.Result;
                    
                    if (User != null)
                    {
                        var profile = new UserProfile{DisplayName = userName};
                        var profileTask = User.UpdateUserProfileAsync(profile);
                        yield return new WaitUntil(() => profileTask.IsCompleted);

                        if (profileTask.Exception != null)
                        {
                            print(profileTask.Exception);
                            var firebaseException = profileTask.Exception.GetBaseException() as FirebaseException;
                            var errorCode = (AuthError)firebaseException!.ErrorCode;
                            warningRegisterText.text = "User name set failed";
                        }
                        else
                        {
                            UIManager.instance.LoginScreen();
                            warningRegisterText.text = "";
                        }
                    }
                }    
            }
        }
    }
}
